import os
import sys

from Crypto.PublicKey import RSA

from Crypto.Cipher import AES
from Crypto.Util import Counter

import binascii
import json
import tarfile

from socket_communication import SecureCommunication
import database_query as dq


# This is the client software that takes a link.10f and gets the according data, decrypts it and saves it to disc

# Link file:
#
# IP
# PORT
# NAME (use this one to get the encrypted file)
# ORIGFILENAME (use this one as a filename for the decrypted file)
# AES (64 hex chars)
#

# Example (with the rsa_pub keys shortened for readability
'''
{"name": "music",
"originalfilename": "Marina Topley Bird Track 01.mp3",
"key": "c389edc914cd375c5367ca50bcc80591f6ef11b18db3ae3c850628d6f9cf5f66", "iv": "0313bc0a41820832406fb28980f4cadd",
"data_rsa_pub": "30820122300d060",

"address_list": 
[ {"rsa_pub": "122300d0609", "ip": "127.0.0.1", "port": 12000}
, {"rsa_pub": "4886f70d011", "ip": "127.0.0.1", "port": "11000"}
, {"rsa_pub": "3006092a864", "ip": "127.0.0.1", "port": "10000"}
]

}
'''


if len(sys.argv) == 1:
    print('Usage:')
    print('')
    print('python3 get_data_from_link.py link.10f')
    print('python3 get_data_from_link.py link.10f dest_folder')
    print('This both works for files and folders')
    # print('python3 get_data_from_link.py --testrun')

    exit()


def get_encrypted_data(ip, port, name, rsa_pub):
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    print('CONNECT:', ip, port)
    s.connect((ip, int(port)))
    query = 'ONEFINGER:' + name + ',' + rsa_pub
    print('Send query:', query)
    print('Query is of type:', type(query))
    if socket_secure_send(s, query):
        print('recv...')
        answer = socket_secure_recv(s, 0, extensive_logging=False, decode=False)
        if answer is not None:
            # print('GOT ANSWER:', answer)
            return answer
            #return bytes.fromhex(answer)
        else:
            print('Unknown answer / error')
            
    # for now fake it:
    #dataaes = None
    #with open('data/test.aes', 'rb') as f:
    #    dataaes = f.read()
    
    return None


# AES supports multiple key sizes: 16 (AES128), 24 (AES192), or 32 (AES256).
key_bytes = 32


# Takes as input a 32-byte key, a 16-byte IV, and a ciphertext, and outputs the
# corresponding plaintext.
def decrypt(key, iv, ciphertext):
    assert len(key) == key_bytes

    # Initialize counter for decryption. iv should be the same as the output of
    # encrypt().
    # iv_int = int(iv.encode('hex'), 16)
    iv_int = int(binascii.hexlify(iv), 16) 
    ctr = Counter.new(AES.block_size * 8, initial_value=iv_int)

    # Create AES-CTR cipher.
    aes = AES.new(key, AES.MODE_CTR, counter=ctr)

    # Decrypt and return the plaintext.
    plaintext = aes.decrypt(ciphertext)
    return plaintext


def chomp(x):
    if x.endswith("\r\n"): return x[:-2]
    if x.endswith("\n") or x.endswith("\r"): return x[:-1]
    return x


def get_data(linkfile, dest_folder):
    # use bytes.fromhex()
    print('get link')

    use_secure_link = True
    if use_secure_link:  # New secure link
        # both name and secure encryption
        with open(linkfile) as json_data:
            print('get json')
            d = json.load(json_data)

            if 'address_list' in d and 'key' in d and 'iv' in d and 'data_rsa_pub' in d:

                address_list = d['address_list']
                # this is the key,iv for the encrypted file we want to retrieve
                key = bytes.fromhex(d['key'])
                iv = bytes.fromhex(d['iv'])
                data_rsa_pub = d['data_rsa_pub']

                # Loop all adresses and try to get the data. todo: make this parallel / or at least bail out faster if node is not up.
                for address in address_list:
                    if 'rsa_pub' in address and 'ip' in address and 'port' in address:
                        rsa_hex_pub_key = address['rsa_pub']
                        ip = address['ip']
                        port = address['port']

                        # Insert nodes from the link
                        if not dq.node_exists(ip, port, rsa_hex_pub_key):
                            dq.insert_node(ip, port, rsa_hex_pub_key)

                        rsa_pub_key = RSA.importKey(bytes.fromhex(rsa_hex_pub_key))
                    else:
                        print('#Error: address corrupt.')

                    print('set up secure link:', ip, port)
                    sys.stdout.flush()
                    sc = SecureCommunication(ip, port, rsa_pub_key)  # rsa_pub here is the one linked to the DATA, not the node we are connecting to!
                    if sc.is_active():
                        print('query for data')
                        sys.stdout.flush()
                        query = 'ONEFINGER:' + d['name'] + ',' + data_rsa_pub  # rsa in plaintext here as we use it as a name
                        encrypted_data = sc.send(query)
                        if encrypted_data is not None:
                            print('Decrypt data')
                            plain_data = decrypt(key, iv, encrypted_data)
                            originalfilename = d['originalfilename']
                            is_gzipped = originalfilename[-7:] == '.tar.gz'
                            print('File extension check:', originalfilename, '=>', originalfilename[-7:], ' result=', is_gzipped)
                            if dest_folder:
                                if not os.path.exists(dest_folder):
                                    os.makedirs(dest_folder)
                                originalfilename = os.path.join(dest_folder, originalfilename)
                            with open(originalfilename, 'wb') as f:
                                f.write(plain_data)
                                f.close()
                                print('Success: retrieved data.')

                                # was it a folder? If so, decompress it here!
                                if is_gzipped:
                                    print('Untar and gunzip')
                                    try:
                                        with tarfile.open(originalfilename, "r:gz") as tar_handle:
                                            dir = dest_folder if dest_folder else './'
                                            tar_handle.extractall(path=dir)  # Will be made here in this folder
                                            tar_handle.close()
                                    except e:
                                        print('An exception occured:', e)

                                return
                        else:
                            print('SecureCommunication closed down too early')
                    else:
                        print('SecureCommunication Failed')
            else:
                print('#Error: Missing information in link file.')

    else:  # old encrypted but sent over weak link
        with open(linkfile) as json_data:
            d = json.load(json_data)  # todo remove chomp!
            key = bytes.fromhex(d['key'])
            iv = bytes.fromhex(d['iv'])
            
            encrypted_data = get_encrypted_data(d['ip'], d['port'], d['name'], d['rsa_pub'])
            print('Decrypt data')
            plain_data = decrypt(key, iv, encrypted_data)

            with open(d['originalfilename'], 'wb') as f:
                f.write(plain_data)

# Test case:
def test():
    print('[get_data_from_link.py.py]')
    print('Test the get_data_from_link:')
    get_data('test.10f')
    get_data('image.10f')


if sys.argv[1] == '--testrun':
    print('--testrun')
    test()
else:
    print('manually get data')
    dest_folder = None
    if len(sys.argv) >= 3:
        dest_folder = sys.argv[2]
    get_data(sys.argv[1], dest_folder)
