import sys
import os

from Crypto.Cipher import AES
from Crypto.Util import Counter
from Crypto import Random
import binascii
import os.path
import tarfile

import database_connection as dc
import database_query as dq

from encryption import CipherHelper

if len(sys.argv) == 1:
    print('Usage:')
    print('To share a file:')
    print('python3 insert_data.py dataname filefolder file')
    print('To share a whole folder:')
    print('python3 insert_data.py -f dataname filefolder')
    # print('python3 insert_data.py --testrun')
    print('')
    print('Example:')
    print('python3 insert_data.py flower home/images flower.jpg')
    print('python3 insert_data.py -f my_website html/websites/my_site')

    exit()

"""
Generates one database entry from a specific file
Database:
NAME = The link-name, can be anything. This name will be used for the retrieval of the data.
ORIGINALFILENAME = the files name when it will be generated from a link, without the path as the path is local.
KEY, IV = AES keypair
PUBKEY = an RSA public key (ours if we are publishing links, from other node if we have done a swicharoo with them and store their data)

if NAME already exists, an update is done, using the already existing AES keypair.
"""


def chomp(x):
    if x.endswith("\r\n"): return x[:-2]
    if x.endswith("\n") or x.endswith("\r"): return x[:-1]
    return x


# This loads and encrypts data, inserts it into the table 'data_owned'
# stores encrypted data in folder '/data/local/' as it is our owned data.
# Later on, the scheduler will ask other nodes for swicharoo (e.g storing our data at their place, and their data at our place in /data/$nodenr$/)
def create_our_data(name, filefolder, file, rsa_pub_key):
    # Insert in database
    
    connection = dc.get_connection()

    print('Insert new data:')

    filepath = os.path.join(filefolder, file)

    data = None
    with open(filepath, 'rb') as f:
        data = f.read()
    
    if data is None:
        print('File not found')
        exit(-3)

    cursor = connection.cursor()
    cursor.execute("SELECT KEY, IV FROM data_owned WHERE NAME = '" + name + "'")
    rows = cursor.fetchall()
    # Does a database entry exists?
    if len(rows) > 0:
        print("#Error entry already exist (insert_data) use refresh_data.py instead")
        exit(0)
        # Get IV, KEY
        #row = rows[0]
        #key = bytes.fromhex(chomp(row[0]))
        #iv = bytes.fromhex(chomp(row[1]))
        #ch = CipherHelper(aes_key=key, raw_iv=iv)
    #else:
        # Generate fresh IV, KEY This calls the system's random number generator (RNG).
    ch = CipherHelper()
        #key = Random.new().read(key_bytes)
        #iv = Random.new().read(AES.block_size)  # Choose a random, 16-byte IV.

    # Encrypt with AES-CTR
    encrypted_data = ch.encrypt(data)

    # Make a 'local' folder if it doesn't already exist
    directory = 'data/local/'
    if not os.path.exists(directory):
        os.makedirs(directory)

    # Store the encypted data in the data folder so listener can serve it to clients:
    with open(directory + name + '.aes', 'wb') as f:
        f.write(encrypted_data)

    # Delete any stale data:
    #cursor.execute("DELETE FROM data_owned WHERE NAME = '" + name + "';")
    #connection.commit()
    
    AEShex = ch.aes_key.hex()
    IVhex = ch.raw_iv.hex()

    cursor.execute("INSERT INTO data_owned(NAME, ORIGINALFILENAME, KEY, IV, PUBKEY) VALUES(?, ?, ?, ?, ?);", (name, file, AEShex, IVhex, rsa_pub_key))
    connection.commit()

    
# Test case:
def test():
    print('[insert_data.py]')
    prv, pub = dq.get_rsa_hex_keys()
    create_our_data('test', 'my_local_data', 'test.txt', pub)
    create_our_data('image', 'my_local_data', 'test.png', pub)



if sys.argv[1] == '--testrun':
    test()
else:
    if len(sys.argv) >= 4:  # Use this to insert a link in the database with the local users rsa_pub_key
        # fetch the users rsa_pub_key
        prv, pub = dq.get_rsa_hex_keys()

        if sys.argv[1] == '-f':
            print('Insert folder')
            regex = '.*'
            if len(sys.argv) == 5:
                regex = sys.argv[4]
            folder = sys.argv[3]
            # get all the files in 'folder' and zip it:
            data_name = sys.argv[2]
            full_data_name = data_name+'.tar.gz'
            with tarfile.open(full_data_name, "w:gz") as tar_handle:
                tar_handle.add(folder)

            print('Insert new folder link')
            # name, filefolder, file, rsa_pub_key
            create_our_data(data_name, './', full_data_name, pub)  # Local owned link
        else:
            print('Insert new link')
            # name, filefolder, file, rsa_pub_key
            create_our_data(sys.argv[1], sys.argv[2], sys.argv[3], pub)  # Local owned link


    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    