# Run this to setup.
# Will not harm existing installations
# Gen erates RSA keypair, database etc.

import sys
import rsa_helper as rh
import generate_database as gdb
import database_connection as dc
import database_query as dq

# For test system
from Crypto.PublicKey import RSA

def setup():
    print('[setup.py]')
    print('Welcome to the 1-2-10 finger setup!')

    # gen database?
    if not dc.database_exists():
        print('setup: generate database')
        gdb.create_software_database()

    # RSA keys exist?
    r = rh.get_rsa_keypair(True)
    if r is None:
        print('setup: generate RSA keys')
        # Generate them:
        prv, pub = rh.generate_rsa_keypair(2048)
        # insert in database
        dq.set_rsa(prv, pub)
        # Save off copy (this can be used for test systems, or for commanding our files from a distance)
        textkey_prv = prv.exportKey('DER').hex()
        with open('secret_key', 'w') as f:
            f.write(textkey_prv)
        textkey_pub = pub.exportKey('DER').hex()
        with open('public_key', 'w') as f:
            f.write(textkey_pub)

    else:
        print('setup: load RSA keys')
        prv, pub = r


if len(sys.argv) == 6 and sys.argv[1] == '--test_add_node':  # second arg is path to public key, third is nodename, 4,5 is ip, port
    # copy key from directory and insert as known node
    file = sys.argv[2]
    nodename = sys.argv[3]
    ip = sys.argv[4]
    port = sys.argv[5]
    print('Load up public key and insert it into our database:')
    with open(file, 'r') as f:
        pub_hex = f.read()
    # pub_key = RSA.importKey(bytes.fromhex(pub_hex))

    # insert into database
    dq.insert_node(ip, port, pub_hex)
else:
    setup()






























